tags = {
        '2011_MagDown_MC_Bd2Ksee': {'conddb':"sim-20160614-1-vc-md100",
                                    'dddb': "dddb-20170721-1", 'id' : '11124101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagUp_MC_Bd2Ksee': {'conddb':"sim-20160614-1-vc-mu100",
                                    'dddb': "dddb-20170721-1", 'id' : '11124101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagDown_MC_Bd2Ksee': {'conddb': "sim-20130522-1-vc-md100",
                                    'dddb': "dddb-20130929-1", 'id' : '11124101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim08'},
        '2012_MagUp_MC_Bd2Ksee': {'conddb': "sim-20130522-1-vc-mu100",
                                    'dddb': "dddb-20130929-1", 'id' : '11124101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim08'},
        '2015_MagUp_MC_Bd2Ksee': {'conddb':"sim-20161124-vc-mu100",
                                    'dddb': "dddb-20170721-3", 'id' : '11124101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagDown_MC_Bd2Ksee': {'conddb':"sim-20161124-vc-md100",
                                    'dddb': "dddb-20170721-3", 'id' : '11124101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagDown_MC_Bd2Ksee': {'conddb':"sim-20161124-2-vc-md100",
                                    'dddb': "dddb-20150724", 'id' : '11124101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagUp_MC_Bd2Ksee': {'conddb':"sim-20161124-2-vc-mu100",
                                    'dddb': "dddb-20150724", 'id' : '11124101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagDown_MC_Bd2Ksee': {'conddb':"sim-20190430-1-vc-md100",
                                    'dddb': "dddb-20170721-3", 'id' : '11124101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagUp_MC_Bd2Ksee': {'conddb':"sim-20190430-1-vc-mu100",
                                    'dddb': "dddb-20170721-3", 'id' : '11124101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagDown_MC_Bd2Ksee': {'conddb':"sim-20190430-vc-md100",
                                    'dddb': "dddb-20170721-3", 'id' : '11124101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagUp_MC_Bd2Ksee': {'conddb':"sim-20190430-vc-mu100",
                                    'dddb': "dddb-20170721-3", 'id' : '11124101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagDown_MC_Bd2KsJpsee': {'conddb':"sim-20130522-vc-md100",
                                    'dddb': "dddb-20130929", 'id' : '11154100',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim08'},
        '2011_MagUp_MC_Bd2KsJpsee': {'conddb':"sim-20130522-vc-mu100",
                                    'dddb': "dddb-20130929", 'id' : '11154100',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim08'},
        '2012_MagDown_MC_Bd2KsJpsee': {'conddb':"sim-20130522-1-vc-md100",
                                    'dddb': "dddb-20130929-1", 'id' : '11154100',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim08'},
        '2012_MagUp_MC_Bd2KsJpsee': {'conddb':"sim-20130522-1-vc-mu100",
                                    'dddb': "dddb-20130929-1", 'id' : '11154100',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim08'},
        '2015_MagUp_MC_Bd2KsJpsee': {'conddb':"sim-20161124-vc-mu100",
                                    'dddb': "dddb-20170721-3", 'id' : '11154100',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagDown_MC_Bd2KsJpsee': {'conddb':"sim-20161124-vc-md100",
                                    'dddb': "dddb-20170721-3", 'id' : '11154100',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagDown_MC_Bd2KsJpsee': {'conddb':"sim-20170721-2-vc-md100",
                                    'dddb': "dddb-20170721-3", 'id' : '11154100',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagUp_MC_Bd2KsJpsee': {'conddb':"sim-20170721-2-vc-mu100",
                                    'dddb': "dddb-20170721-3", 'id' : '11154100',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagDown_MC_Bd2KsJpsee': {'conddb':"sim-20190430-1-vc-md100",
                                    'dddb': "dddb-20170721-3", 'id' : '11154100',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagUp_MC_Bd2KsJpsee': {'conddb':"sim-20190430-1-vc-mu100",
                                    'dddb': "dddb-20170721-3", 'id' : '11154100',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagDown_MC_Bd2KsJpsee': {'conddb':"sim-20190430-vc-md100",
                                    'dddb': "dddb-20170721-3", 'id' : '11154100',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagUp_MC_Bd2KsJpsee': {'conddb':"sim-20190430-vc-mu100",
                                    'dddb': "dddb-20170721-3", 'id' : '11154100',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagDown_MC_Bd2KsJpsmm': {'conddb':"sim-20160614-1-vc-md100",
                                    'dddb': "dddb-20160318-1", 'id' : '11144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagUp_MC_Bd2KsJpsmm': {'conddb':"sim-20160614-1-vc-mu100",
                                    'dddb': "dddb-20160318-1", 'id' : '11144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagDown_MC_Bd2KsJpsmm': {'conddb':"sim-20160321-2-vc-md100",
                                    'dddb': "dddb-20150928", 'id' : '11144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagUp_MC_Bd2KsJpsmm': {'conddb':"sim-20160321-2-vc-mu100",
                                    'dddb': "dddb-20150928", 'id' : '11144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagDown_MC_Bd2KsJpsmm': {'conddb':"sim-20161124-vc-md100",
                                    'dddb': "dddb-20150724", 'id' : '11144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagUp_MC_Bd2KsJpsmm': {'conddb':"sim-20161124-vc-mu100",
                                    'dddb': "dddb-20150724", 'id' : '11144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagDown_MC_Bd2KsJpsmm': {'conddb':"sim-20161124-2-vc-md100",
                                    'dddb': "dddb-20150724", 'id' : '11144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagUp_MC_Bd2KsJpsmm': {'conddb':"sim-20161124-2-vc-mu100",
                                    'dddb': "dddb-20150724", 'id' : '11144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagDown_MC_Bd2KsJpsmm': {'conddb':"sim-20190430-1-vc-md100",
                                    'dddb': "dddb-20170721-3", 'id' : '11144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagUp_MC_Bd2KsJpsmm': {'conddb':"sim-20190430-1-vc-mu100",
                                    'dddb': "dddb-20170721-3", 'id' : '11144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagDown_MC_Bd2KsJpsmm': {'conddb':"sim-20190430-vc-md100",
                                    'dddb': "dddb-20170721-3", 'id' : '11144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagUp_MC_Bd2KsJpsmm': {'conddb':"sim-20190430-vc-mu100",
                                    'dddb': "dddb-20170721-3", 'id' : '11144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagDown_MC_Bd2Ksmm': {'conddb':"sim-20160614-1-vc-md100",
                                    'dddb': "dddb-20170721-1", 'id' : '11114101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagUp_MC_Bd2Ksmm': {'conddb':"sim-20160614-1-vc-mu100",
                                    'dddb': "dddb-20170721-1",'id' : '11114101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagUp_MC_Bd2Ksmm': {'conddb':"Sim08-20130503-1-vc-mu100",
                                    'dddb': "Sim08-20130503-1",'id' : '11114101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim08'},
        '2012_MagDown_MC_Bd2Ksmm': {'conddb':"Sim08-20130503-1-vc-mu100",
                                    'dddb': "Sim08-20130503-1",'id' : '11114101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim08'},
        '2015_MagUp_MC_Bd2Ksmm': {'conddb':"sim-20161124-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '11114101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagDown_MC_Bd2Ksmm': {'conddb':"sim-20161124-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '11114101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagDown_MC_Bd2Ksmm': {'conddb':"sim-20170721-2-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '11114101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagUp_MC_Bd2Ksmm': {'conddb':"sim-20170721-2-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '11114101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagDown_MC_Bd2Ksmm': {'conddb':"sim-20190430-1-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '11114101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagUp_MC_Bd2Ksmm': {'conddb':"sim-20190430-1-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '11114101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagDown_MC_Bd2Ksmm': {'conddb':"sim-20190430-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '11114101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagUp_MC_Bd2Ksmm': {'conddb':"sim-20190430-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '11114101',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagDown_MC_Bd2KsPsimm': {'conddb':"sim-20130522-vc-md100",
                                    'dddb': "dddb-20130929",'id' : '11144110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim08'},
        '2011_MagUp_MC_Bd2KsPsimm': {'conddb':"sim-20130522-vc-mu100",
                                    'dddb': "dddb-20130929",'id' : '11144110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim08'},
        '2012_MagDown_MC_Bd2KsPsimm': {'conddb':"sim-20130522-1-vc-md100",
                                    'dddb': "dddb-20130929",'id' : '11144110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim08'},
        '2012_MagUp_MC_Bd2KsPsimm': {'conddb':"sim-20130522-vc-mu100",
                                    'dddb': "dddb-20130929",'id' : '11144110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim08'},
        '2015_MagDown_MC_Bd2KsPsimm': {'conddb':"sim-20161124-vc-md100",
                                    'dddb': "dddb-20150724",'id' : '11144110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagUp_MC_Bd2KsPsimm': {'conddb':"sim-20161124-vc-mu100",
                                    'dddb': "dddb-20150724",'id' : '11144110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagDown_MC_Bd2KsPsimm': {'conddb':"sim-20170721-2-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '11144110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagUp_MC_Bd2KsPsimm': {'conddb':"sim-20170721-2-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '11144110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagDown_MC_Bd2KsPsimm': {'conddb':"sim-20190430-1-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '11144110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagUp_MC_Bd2KsPsimm': {'conddb':"sim-20190430-1-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '11144110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagDown_MC_Bd2KsPsimm': {'conddb':"sim-20190430-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '11144110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagUp_MC_Bd2KsPsimm': {'conddb':"sim-20190430-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '11144110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagDown_MC_Bd2KsPsiee': {'conddb':"sim-20160614-1-vc-md100",
                                    'dddb': "dddb-20170721-1",'id' : '11154110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagUp_MC_Bd2KsPsiee': {'conddb':"sim-20160614-1-vc-mu100",
                                    'dddb': "dddb-20170721-1",'id' : '11154110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagDown_MC_Bd2KsPsiee': {'conddb':"sim-20160321-2-vc-md100",
                                    'dddb': "dddb-20170721-2",'id' : '11154110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagUp_MC_Bd2KsPsiee': {'conddb':"sim-20160321-2-vc-mu100",
                                    'dddb': "dddb-20170721-2",'id' : '11154110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagDown_MC_Bd2KsPsiee': {'conddb':"sim-20161124-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '11154110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagUp_MC_Bd2KsPsiee': {'conddb':"sim-20161124-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '11154110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagDown_MC_Bd2KsPsiee': {'conddb':"sim-20170721-2-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '11154110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagUp_MC_Bd2KsPsiee': {'conddb':"sim-20170721-2-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '11154110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagDown_MC_Bd2KsPsiee': {'conddb':"sim-20190430-1-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '11154110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagUp_MC_Bd2KsPsiee': {'conddb':"sim-20190430-1-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '11154110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagDown_MC_Bd2KsPsiee': {'conddb':"sim-20190430-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '11154110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagUp_MC_Bd2KsPsiee': {'conddb':"sim-20190430-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '11154110',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},

        '2011_MagDown_MC_Lb2Lee': {'conddb':"sim-20160614-1-vc-md100",
                                    'dddb': "dddb-20170721-1",'id' : '15124102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagUp_MC_Lb2Lee': {'conddb':"sim-20160614-1-vc-mu100",
                                    'dddb': "dddb-20170721-1",'id' : '15124102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagDown_MC_Lb2Lee': {'conddb':"sim-20160321-2-vc-md100",
                                    'dddb': "dddb-20170721-2",'id' : '15124102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagUp_MC_Lb2Lee': {'conddb':"sim-20160321-2-vc-mu100",
                                    'dddb': "dddb-20170721-2",'id' : '15124102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagDown_MC_Lb2Lee': {'conddb':"sim-20161124-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15124102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagUp_MC_Lb2Lee': {'conddb':"sim-20161124-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15124102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagDown_MC_Lb2Lee': {'conddb':"sim-20170721-2-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15124102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagUp_MC_Lb2Lee': {'conddb':"sim-20170721-2-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15124102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagDown_MC_Lb2Lee': {'conddb':"sim-20190430-1-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15124102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagUp_MC_Lb2Lee': {'conddb':"sim-20190430-1-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15124102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagDown_MC_Lb2Lee': {'conddb':"sim-20190430-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15124102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagUp_MC_Lb2Lee': {'conddb':"sim-20190430-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15124102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagDown_MC_Lb2Lem': {'conddb':"sim-20160614-1-vc-md100",
                                    'dddb': "dddb-20170721-1",'id' : '15114113',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagUp_MC_Lb2Lem': {'conddb':"sim-20160614-1-vc-mu100",
                                    'dddb': "dddb-20170721-1",'id' : '15114113',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagDown_MC_Lb2Lem': {'conddb':"sim-20160321-2-vc-md100",
                                    'dddb': "dddb-20170721-2",'id' : '15114113',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagUp_MC_Lb2Lem': {'conddb':"sim-20160321-2-vc-mu100",
                                    'dddb': "dddb-20170721-2",'id' : '15114113',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagDown_MC_Lb2Lem': {'conddb':"sim-20161124-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15114113',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagUp_MC_Lb2Lem': {'conddb':"sim-20161124-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15114113',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagDown_MC_Lb2Lem': {'conddb':"sim-20170721-2-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15114113',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagUp_MC_Lb2Lem': {'conddb':"sim-20170721-2-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15114113',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagDown_MC_Lb2Lem': {'conddb':"sim-20190430-1-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15114113',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagUp_MC_Lb2Lem': {'conddb':"sim-20190430-1-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15114113',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagDown_MC_Lb2Lem': {'conddb':"sim-20190430-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15114113',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagUp_MC_Lb2Lem': {'conddb':"sim-20190430-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15114113',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagDown_MC_Lb2Lmm': {'conddb':"sim-20160614-1-vc-md100",
                                    'dddb': "dddb-20170721-1",'id' : '15114102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagUp_MC_Lb2Lmm': {'conddb':"sim-20160614-1-vc-mu100",
                                    'dddb': "dddb-20170721-1",'id' : '15114102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagDown_MC_Lb2Lmm': {'conddb':"sim-20160321-2-vc-md100",
                                    'dddb': "dddb-20170721-2",'id' : '15114102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagUp_MC_Lb2Lmm': {'conddb':"sim-20160321-2-vc-mu100",
                                    'dddb': "dddb-20170721-2",'id' : '15114102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagDown_MC_Lb2Lmm': {'conddb':"sim-20161124-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15114102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagUp_MC_Lb2Lmm': {'conddb':"sim-20161124-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15114102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagDown_MC_Lb2Lmm': {'conddb':"sim-20170721-2-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15114102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagUp_MC_Lb2Lmm': {'conddb':"sim-20170721-2-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15114102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagDown_MC_Lb2Lmm': {'conddb':"sim-20190430-1-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15114102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagUp_MC_Lb2Lmm': {'conddb':"sim-20190430-1-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15114102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagDown_MC_Lb2Lmm': {'conddb':"sim-20190430-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15114102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagUp_MC_Lb2Lmm': {'conddb':"sim-20190430-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15114102',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagDown_MC_Lb2LJpsmm': {'conddb':"sim-20160614-1-vc-md100",
                                    'dddb': "dddb-20170721-1",'id' : '15144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim08'},
        '2011_MagUp_MC_Lb2LJpsmm': {'conddb':"sim-20160614-1-vc-mu100",
                                    'dddb': "dddb-20170721-1",'id' : '15144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim08'},
        '2012_MagDown_MC_Lb2LJpsmm': {'conddb':"sim-20160321-2-vc-md100",
                                    'dddb': "dddb-20170721-2",'id' : '15144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim08'},
        '2012_MagUp_MC_Lb2LJpsmm': {'conddb':"sim-20160321-2-vc-mu100",
                                    'dddb': "dddb-20170721-2",'id' : '15144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim08'},
        '2015_MagDown_MC_Lb2LJpsmm': {'conddb':"sim-20161124-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagUp_MC_Lb2LJpsmm': {'conddb':"sim-20161124-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagDown_MC_Lb2LJpsmm': {'conddb':"sim-20170721-2-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagUp_MC_Lb2LJpsmm': {'conddb':"sim-20170721-2-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagDown_MC_Lb2LJpsmm': {'conddb':"sim-20190430-1-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagUp_MC_Lb2LJpsmm': {'conddb':"sim-20190430-1-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagDown_MC_Lb2LJpsmm': {'conddb':"sim-20190430-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagUp_MC_Lb2LJpsmm': {'conddb':"sim-20190430-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15144103',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagDown_MC_Lb2LJpsee': {'conddb':"sim-20160614-1-vc-md100",
                                    'dddb': "dddb-20170721-1",'id' : '15154105',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagUp_MC_Lb2LJpsee': {'conddb':"sim-20160614-1-vc-mu100",
                                    'dddb': "dddb-20170721-1",'id' : '15154105',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagDown_MC_Lb2LJpsee': {'conddb':"sim-20160321-2-vc-md100",
                                    'dddb': "dddb-20170721-2",'id' : '15154105',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagUp_MC_Lb2LJpsee': {'conddb':"sim-20160321-2-vc-mu100",
                                    'dddb': "dddb-20170721-2",'id' : '15154105',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagDown_MC_Lb2LJpsee': {'conddb':"sim-20161124-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15154105',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagUp_MC_Lb2LJpsee': {'conddb':"sim-20161124-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15154105',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagDown_MC_Lb2LJpsee': {'conddb':"sim-20170721-2-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15154105',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagUp_MC_Lb2LJpsee': {'conddb':"sim-20170721-2-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15154105',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagDown_MC_Lb2LJpsee': {'conddb':"sim-20190430-1-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15154105',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagUp_MC_Lb2LJpsee': {'conddb':"sim-20190430-1-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15154105',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagDown_MC_Lb2LJpsee': {'conddb':"sim-20190430-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15154105',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagUp_MC_Lb2LJpsee': {'conddb':"sim-20190430-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15154105',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagDown_MC_Lb2LPsiee': {'conddb':"sim-20160614-1-vc-md100",
                                    'dddb': "dddb-20170721-1",'id' : '15154115',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagUp_MC_Lb2LPsiee': {'conddb':"sim-20160614-1-vc-mu100",
                                    'dddb': "dddb-20170721-1",'id' : '15154115',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagDown_MC_Lb2LPsiee': {'conddb':"sim-20160321-2-vc-md100",
                                    'dddb': "dddb-20170721-2",'id' : '15154115',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagUp_MC_Lb2LPsiee': {'conddb':"sim-20160321-2-vc-mu100",
                                    'dddb': "dddb-20170721-2",'id' : '15154115',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagDown_MC_Lb2LPsiee': {'conddb':"sim-20161124-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15154115',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagUp_MC_Lb2LPsiee': {'conddb':"sim-20161124-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15154115',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagDown_MC_Lb2LPsiee': {'conddb':"sim-20170721-2-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15154115',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagUp_MC_Lb2LPsiee': {'conddb':"sim-20170721-2-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15154115',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagDown_MC_Lb2LPsiee': {'conddb':"sim-20190430-1-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15154115',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagUp_MC_Lb2LPsiee': {'conddb':"sim-20190430-1-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15154115',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagDown_MC_Lb2LPsiee': {'conddb':"sim-20190430-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15154115',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagUp_MC_Lb2LPsiee': {'conddb':"sim-20190430-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15154115',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagDown_MC_Lb2LPsimm': {'conddb':"sim-20160614-1-vc-md100",
                                    'dddb': "dddb-20170721-1",'id' : '15144111',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagUp_MC_Lb2LPsimm': {'conddb':"sim-20160614-1-vc-mu100",
                                    'dddb': "dddb-20170721-1",'id' : '15144111',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagDown_MC_Lb2LPsimm': {'conddb':"sim-20160321-2-vc-md100",
                                    'dddb': "dddb-20170721-2",'id' : '15144111',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagUp_MC_Lb2LPsimm': {'conddb':"sim-20160321-2-vc-mu100",
                                    'dddb': "dddb-20170721-2",'id' : '15144111',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagDown_MC_Lb2LPsimm': {'conddb':"sim-20161124-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15144111',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagUp_MC_Lb2LPsimm': {'conddb':"sim-20161124-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15144111',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagDown_MC_Lb2LPsimm': {'conddb':"sim-20170721-2-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15144111',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagUp_MC_Lb2LPsimm': {'conddb':"sim-20170721-2-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15144111',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagDown_MC_Lb2LPsimm': {'conddb':"sim-20190430-1-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15144111',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagUp_MC_Lb2LPsimm': {'conddb':"sim-20190430-1-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15144111',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagDown_MC_Lb2LPsimm': {'conddb':"sim-20190430-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15144111',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagUp_MC_Lb2LPsimm': {'conddb':"sim-20190430-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15144111',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagDown_MC_Bu2JpsiK_mm': {'conddb':"sim-20160614-1-vc-md100",
                                    'dddb': "dddb-20170721-1",'id' : '12143001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagUp_MC_Bu2JpsiK_mm': {'conddb':"sim-20160614-1-vc-mu100",
                                    'dddb': "dddb-20170721-1",'id' : '12143001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagDown_MC_Bu2JpsiK_mm': {'conddb':"sim-20160321-2-vc-md100",
                                    'dddb': "dddb-20170721-2",'id' : '12143001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagUp_MC_Bu2JpsiK_mm': {'conddb':"sim-20160321-2-vc-mu100",
                                    'dddb': "dddb-20170721-2",'id' : '12143001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagDown_MC_Bu2JpsiK_mm': {'conddb':"sim-20161124-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '12143001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagUp_MC_Bu2JpsiK_mm': {'conddb':"sim-20161124-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '12143001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagDown_MC_Bu2JpsiK_mm': {'conddb':"sim-20170721-2-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '12143001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagUp_MC_Bu2JpsiK_mm': {'conddb':"sim-20170721-2-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '12143001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagDown_MC_Bu2JpsiK_mm': {'conddb':"sim-20190430-1-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '12143001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagUp_MC_Bu2JpsiK_mm': {'conddb':"sim-20190430-1-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '12143001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagDown_MC_Bu2JpsiK_mm': {'conddb':"sim-20190430-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '12143001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagUp_MC_Bu2JpsiK_mm': {'conddb':"sim-20190430-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '12143001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagDown_MC_Bu2JpsiK_ee': {'conddb':"sim-20160614-1-vc-md100",
                                    'dddb': "dddb-20170721-1",'id' : '12153001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2011_MagUp_MC_Bu2JpsiK_ee': {'conddb':"sim-20160614-1-vc-mu100",
                                    'dddb': "dddb-20170721-1",'id' : '12153001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagDown_MC_Bu2JpsiK_ee': {'conddb':"sim-20160321-2-vc-md100",
                                    'dddb': "dddb-20170721-2",'id' : '12153001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2012_MagUp_MC_Bu2JpsiK_ee': {'conddb':"sim-20160321-2-vc-mu100",
                                    'dddb': "dddb-20170721-2",'id' : '12153001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagDown_MC_Bu2JpsiK_ee': {'conddb':"sim-20161124-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '12153001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2015_MagUp_MC_Bu2JpsiK_ee': {'conddb':"sim-20161124-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '12153001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagDown_MC_Bu2JpsiK_ee': {'conddb':"sim-20170721-2-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '12153001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagUp_MC_Bu2JpsiK_ee': {'conddb':"sim-20170721-2-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '12153001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagDown_MC_Bu2JpsiK_ee': {'conddb':"sim-20190430-1-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '12153001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2017_MagUp_MC_Bu2JpsiK_ee': {'conddb':"sim-20190430-1-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '12153001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagDown_MC_Bu2JpsiK_ee': {'conddb':"sim-20190430-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '12153001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2018_MagUp_MC_Bu2JpsiK_ee': {'conddb':"sim-20190430-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '12153001',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09'},
        '2016_MagDown_MC_Lb2Lcenu': {'conddb':"sim-20170721-2-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15574121',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09c'},
        '2016_MagUp_MC_Lb2Lcenu': {'conddb':"sim-20170721-2-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15574121',
                                    'StrippingLine': "Bu2LLK_eeLine2",
                                    'StrippingStream': "AllStreams", 'sim':'Sim09c'},
        '2016_MagDown_MC_Lb2Lcmnu': {'conddb':"sim-20170721-2-vc-md100",
                                    'dddb': "dddb-20170721-3",'id' : '15574142',
                                    'StrippingLine': "Bu2LLKNoPID_eeLine2",
                                    'StrippingStream': "Bu2LLK_NoPID.Strip", 'sim':'Sim09d'},
        '2016_MagUp_MC_Lb2Lcmnu': {'conddb':"sim-20170721-2-vc-mu100",
                                    'dddb': "dddb-20170721-3",'id' : '15574142',
                                    'StrippingLine': "Bu2LLKNoPID_eeLine2",
                                    'StrippingStream': "Bu2LLK_NoPID.Strip", 'sim':'Sim09d'},

        }



from Configurables import DaVinci
try:
    DaVinci().Turbo = False
except AttributeError:
    # Older DaVinci versions don't support Turbo at all
    pass

DaVinci().InputType = 'DST'
DaVinci().DataType = '2016'
DaVinci().Simulation = True
DaVinci().Lumi = False
DaVinci().DDDBtag = 'dddb-20170721-3'
DaVinci().CondDBtag = 'sim-20170721-2-vc-md100'


import os

from Configurables import (
    DaVinci,
    GaudiSequencer,
    DecayTreeTuple,
    MCDecayTreeTuple,
    TupleToolDecay
)

from PhysSelPython.Wrappers import (
    AutomaticData,
    FilterSelection,
    SelectionSequence,
)
from DecayTreeTuple.Configuration import *

from related_info import getLoKiTool
import tags
import locations
from cbtesting.helpers import decay_descriptors 

def tuple_maker(tuple_name,upstream_electrons):
    """Return a sequence for producing ntuples.

    Assumes the decay structure is like:

        Beauty -> (J/Psi -> l- l+) (Lambda0 -> p pi-)
    """
    # FIRST OBTAIN SOME INFORMATION FROM DAVINCI
    year = DaVinci().DataType
    is_mc = DaVinci().Simulation
    is_mdst = (DaVinci().InputType.upper() == 'MDST')

    # NOW DETERMINE STREAM NAME
    # The sample name is formatted as {year}_{polarity}_MC_{decay}  ; needed for stripping line names
    if is_mc:
        sample_decay   = DaVinci().TupleFile
        #decay = 'Lb2LJpsmm' # THIS WAS USED ONLY FOR TESTING BY HAND 
        sample_name = year+"_MagUp" # Note that polarity does not matter for line name
        print("TEST ", sample_decay)
        if is_mc:     sample_name += "_MC_"+sample_decay 
    else:
        sample_name = None # Sample name not needed for Data

    print("Test2 ", sample_name) 
    # Determine stream and set tags if needed
    if is_mc:     
        sample_settings = tags.tags[sample_name]
        stream = sample_settings['StrippingStream']
        DaVinci().CondDBtag = sample_settings['conddb']
        DaVinci().DDDBtag   = sample_settings['dddb']
    if not is_mc: 
        stream = 'Leptonic'

    # NOW DETERMINE STRIPPING LINE
    # Get which two leptons are in final state
    # tuple_name definitions are Lb2JpsiL_llTuple, Lb2LemuTuple,Bu2JpsiK_llTuple,Lb2Lee_SSTuple  
    # so replace Lb2L by _, split at '_', take second part and slice first two particles
    # this returns mm, ee or em; 
    lepton_mode = tuple_name.replace('Lb2L','_').split('_')[1][:2]
    if 'em' in lepton_mode: lepton_mode = 'me' #need 'me' for stripping line name
    if 'SS' in tuple_name:  lepton_mode += 'SS'

    print ('Lepton_mode',lepton_mode)

    # Combine stream and stripping line name depending on case to return locations
    location_list = locations.get_locations(stream,lepton_mode,sample_name,is_mc,is_mdst,upstream_electrons=upstream_electrons)    
    print(location_list)

    # FOR LOADING DECAY DESCRIPTORS, SETTING BRANCHES, ETC.
    decay_rec                 = tuple_name.replace('Tuple','') # DECAY NAME
    decay_desc_full           = decay_descriptors.dict_rec[decay_rec]
    tuple_branches, part_list = decay_descriptors.tuple_branches(decay_desc_full)
    print ('Decay rec',decay_rec)   
    print ('Decay desc',decay_desc_full)   
    
    dtt = DecayTreeTuple(tuple_name)
    dtt.Inputs = location_list
    dtt.Decay  = decay_desc_full
     


    dtt.addBranches(tuple_branches)

    
    # OBTAIN PARTICLE NAMES FOR ASSIGNING TUPLETOOLS 
    if 'Lb' in decay_rec:    
        b_hadr     = [dtt.Lb]
        resonances = [dtt.JPs,dtt.L0]
        hadrons    = [dtt.P ,dtt.Pi]
        leptons    = [dtt.L1,dtt.L2]  
        dtf_parts  = ['Lambda_b0','Lambda0']
        dtf_label  = ['Lb'       ,'L0'     ]

    if 'Bd' in decay_rec:    
        b_hadr     = [dtt.B]
        resonances = [dtt.JPs,dtt.K0]
        hadrons    = [dtt.Pi1,dtt.Pi2]
        leptons    = [dtt.L1,dtt.L2]  
        dtf_parts  = ['B0'       ,'KS0'    ]
        dtf_label  = ['B0'       ,'K0'     ]

    if 'Bu' in decay_rec:
        b_hadr     = [dtt.B]
        resonances = [dtt.JPs]
        hadrons    = [dtt.K]
        leptons    = [dtt.L1,dtt.L2]  
        dtf_parts  = ['B+']
        dtf_label  = ['B']

    

    particles  = b_hadr+resonances+hadrons+leptons    
    print (b_hadr[0])
    print (particles[0])
    print (b_hadr==particles[0])


    # NOTE: DEFAULT TOOLS INCLUDED ALWAYS so this is just included for documentation
    default_tools = [
        "TupleToolANNPID",
        "TupleToolGeometry",
        "TupleToolPid",
        "TupleToolEventInfo",
        #"TupleToolKinematic",
        ]
    # Tool list
    specific_tools = [
        "TupleToolPropertime",
    ]
    event_wide_tools = [
        "TupleToolL0Data",
        "TupleToolPrimaries",
        "TupleToolRecoStats",
        "TupleToolTrackInfo",
    ]
   
    for particle in particles:
        kin_tool = particle.addTupleTool("TupleToolKinematic")
        kin_tool.Verbose = True # To include AtVtx_P, which gives better resolution for DD tracks

    for tool in specific_tools:
        for particle in particles:
            particle.addTupleTool(tool)
    for tool in event_wide_tools:
        dtt.addTupleTool(tool)

    # Trigger lines
    l0_triggers = ["L0HadronDecision", "L0ElectronDecision",
                   'L0MuonDecision','L0DiMuonDecision','L0PhotonDecision']

    hlt1_muon       = [ "Hlt1TrackMuonDecision", "Hlt1SingleMuonDecision" ,
                        "Hlt1SingleMuonHighPTDecision" ]
    hlt1_muon_run2  = [ "Hlt1TrackMuonMVADecision"]
    hlt1_dimuon     = [ "Hlt1DiMuonHighMassDecision"    ]
 
    hlt1_track_run1 = [ "Hlt1TrackAllL0Decision", "Hlt1TrackAllL0TightDecision" ]
    hlt1_track_run2 = [ "Hlt1TrackMVADecision", "Hlt1TrackMVALooseDecision",
                        "Hlt1TwoTrackMVADecision", "Hlt1TwoTrackMVALooseDecision"]

    hlt2_muon       = ["Hlt2SingleMuonDecision"]
    hlt2_electron  = ["Hlt2SingleElectronTFHighPtDecision",
                       "Hlt2SingleElectronTFLowPtDecision"]
    hlt2_dimuon     = [ "Hlt2DiMuonDetachedDecision", "Hlt2DiMuonDetachedHeavyDecision",
            "Hlt2DiMuonDetachedJPsiDecision", "Hlt2DiMuonDetachedPsi2SDecision",
            "Hlt2DiMuonJPsiHighPTDecision", "Hlt2DiMuonPsi2SHighPTDecision"         ]

    hlt2_topo_run1  = [ "Hlt2Topo2BodyBBDTDecision", "Hlt2Topo3BodyBBDTDecision"]
    #            "Hlt2Topo4BodyBBDTDecision"]
    hlt2_topomu_run1   = [line.replace('Topo','TopoMu'   ) for line in hlt2_topo_run1]
    hlt2_topoe_run1    = [line.replace('Topo','TopoE'    ) for line in hlt2_topo_run1]
    hlt2_topo_run2     = [line.replace('BBDT',''         ) for line in hlt2_topo_run1]
    hlt2_topomu_run2   = [line.replace('Topo','TopoMu'   ) for line in hlt2_topo_run2]
    hlt2_topoe_run2    = [line.replace('Topo','TopoE'    ) for line in hlt2_topo_run2]
    hlt2_topomumu_run2 = [line.replace('Topo','TopoMuMu' ) for line in hlt2_topo_run2]
    hlt2_topoee_run2   = [line.replace('Topo','TopoEE'   ) for line in hlt2_topo_run2]
    hlt2_topoemu_run2  = [line.replace('Topo','TopoMuE'  ) for line in hlt2_topo_run2]

     # L0 lines
    from Configurables import TupleToolTISTOS
    for (i,particle) in enumerate(particles):
        particle.addTool( TupleToolTISTOS, name = "L0TISTOS"+str(i) )
        particle.ToolList += [ "TupleToolTISTOS/L0TISTOS"+str(i) ]

        # Hlt1 info for each particle to calibrate TrackMVA
        particle.addTool( TupleToolTISTOS, name = "HltTISTOS"+str(i) )
        particle.ToolList += [ "TupleToolTISTOS/HltTISTOS"+str(i) ]

    L0TISTOS_list = [ particles[0].L0TISTOS0,particles[1].L0TISTOS1,particles[2].L0TISTOS2,particles[3].L0TISTOS3,particles[4].L0TISTOS4, ]
    if len(particles) == 7: L0TISTOS_list += [particles[5].L0TISTOS5, particles[6].L0TISTOS6 ]
    HltTISTOS_list = [ particles[0].HltTISTOS0,particles[1].HltTISTOS1,particles[2].HltTISTOS2,particles[3].HltTISTOS3,particles[4].HltTISTOS4, ]
    if len(particles) == 7: HltTISTOS_list += [particles[5].HltTISTOS5, particles[6].HltTISTOS6 ]

    for (i,particle) in enumerate(particles):
        L0TISTOS_list[i].TriggerList = l0_triggers 
        L0TISTOS_list[i].Verbose = True



        # Start with empty list
        HltTISTOS_list[i].TriggerList = []

        # Add the correct trigger lines for the year and channel
        if year in ['2011', '2012']:
            HltTISTOS_list[i].TriggerList += hlt1_track_run1
        else:
            HltTISTOS_list[i].TriggerList += hlt1_track_run2            

        HltTISTOS_list[i].Verbose = True

        if particle in particles[:2]+particles[-2:]: # For Lb, Jpsi, and leptons
            if 'm' in lepton_mode:
                HltTISTOS_list[i].TriggerList += hlt1_muon+hlt2_muon
                if year in ['2016', '2017', '2018']:
                    HltTISTOS_list[i].TriggerList += hlt1_muon_run2
            if 'e' in lepton_mode:
                HltTISTOS_list[i].TriggerList += hlt2_electron

        if particle in particles[:2]: # For Lb and Jpsi
            # For all years
            if 'mm' in lepton_mode:
                HltTISTOS_list[i].TriggerList += hlt1_dimuon
                HltTISTOS_list[i].TriggerList += hlt2_dimuon

            # Add the correct trigger lines based per year and decay channel
            if year in ['2011', '2012']:
                HltTISTOS_list[i].TriggerList += hlt2_topo_run1
                if 'm' in lepton_mode:
                    HltTISTOS_list[i].TriggerList += hlt2_topomu_run1
                if 'e' in lepton_mode:
                    HltTISTOS_list[i].TriggerList += hlt2_topoe_run1

            else:  
                HltTISTOS_list[i].TriggerList += hlt2_topo_run2

                # 2016, 2017, 2018 specific
                if year in ['2016','2017','2018']:
                    if 'mm' in lepton_mode:
                        HltTISTOS_list[i].TriggerList += hlt2_topomu_run2+hlt2_topomumu_run2
                    if 'me' in lepton_mode: 
                        HltTISTOS_list[i].TriggerList += hlt2_topomu_run2+hlt2_topoe_run2+hlt2_topoemu_run2
                    if 'ee' in lepton_mode:
                        HltTISTOS_list[i].TriggerList += hlt2_topoe_run2+hlt2_topoee_run2 



    # Include TupleToolL0Calo for correction of ET cut in 2016 samples + possibility to bin in CaloRegions
    from Configurables import TupleToolL0Calo
    for particle in leptons+hadrons: 
        particle.addTool(TupleToolL0Calo,name='L0Calo_HCAL')
        particle.ToolList += [ "TupleToolL0Calo/L0Calo_HCAL"]
        particle.L0Calo_HCAL.WhichCalo = "HCAL"

    for particle in leptons+hadrons:
        particle.addTool(TupleToolL0Calo,name='L0Calo_ECAL')
        particle.ToolList += [ "TupleToolL0Calo/L0Calo_ECAL"]
        particle.L0Calo_ECAL.WhichCalo = "ECAL"

    # Different DecayTreeFitter setups
    from Configurables import TupleToolDecayTreeFitter
    dtf_name_bhad_pv = "DTF_{0}_PV".format(dtf_label[0]) 

    particles[0].addTool( TupleToolDecayTreeFitter, name = "DTF" )
    particles[0].ToolList += [ "TupleToolDecayTreeFitter/DTF" ]

    particles[0].addTool( particles[0].DTF.clone( "DTF_PV",
                                          Verbose = True,
                                          constrainToOriginVertex = True ) )
    particles[0].ToolList += [ "TupleToolDecayTreeFitter/DTF_PV" ]

    particles[0].addTool( particles[0].DTF.clone( dtf_name_bhad_pv,
                                          Verbose = True,
                                          constrainToOriginVertex = True,
                                          daughtersToConstrain = [dtf_parts[0]] ) )
    particles[0].ToolList += [ "TupleToolDecayTreeFitter/"+dtf_name_bhad_pv ]

    psi2s_sub =  {tuple_branches['JPs']:"psi(2S)"}

    if len(dtf_label) == 2:
        dtf_name_hadr_pv = "DTF_{0}_PV".format(dtf_label[1]) 
        dtf_name_bhad_hadr_pv = "DTF_{0}_{1}_PV".format(dtf_label[0],dtf_label[1]) 
        dtf_name_hadr_jpsi_pv = "DTF_{0}_JPs_PV".format(dtf_label[1]) 
        dtf_name_hadr_psi_pv = "DTF_{0}_Psi_PV".format(dtf_label[1]) 
       
        particles[0].addTool( particles[0].DTF.clone( dtf_name_hadr_pv,
                                              Verbose = True,
                                              constrainToOriginVertex = True,
                                              daughtersToConstrain = [dtf_parts[1]] ) )
        particles[0].ToolList += [ "TupleToolDecayTreeFitter/"+dtf_name_hadr_pv]

        particles[0].addTool( particles[0].DTF.clone( dtf_name_bhad_hadr_pv,
                                              Verbose = True,
                                              constrainToOriginVertex = True,
                                              daughtersToConstrain = dtf_parts ) )
        particles[0].ToolList += [ "TupleToolDecayTreeFitter/"+dtf_name_bhad_hadr_pv ]


        particles[0].addTool( particles[0].DTF.clone( dtf_name_hadr_jpsi_pv,
                                              Verbose = True,
                                              constrainToOriginVertex = True,
                                              daughtersToConstrain = [dtf_parts[1]]+["J/psi(1S)" ] ) )
        particles[0].ToolList += [ "TupleToolDecayTreeFitter/"+dtf_name_hadr_jpsi_pv ]

        particles[0].addTool( particles[0].DTF.clone( dtf_name_hadr_psi_pv,
                                              Verbose = True,
                                              constrainToOriginVertex = True,
                                              daughtersToConstrain = [dtf_parts[1]]+["psi(2S)" ],
                                              Substitutions = psi2s_sub) )
        particles[0].ToolList += [ "TupleToolDecayTreeFitter/"+dtf_name_hadr_psi_pv ]

    else:
        dtf_name_hadr_jpsi_pv = 'DTF_JPs_PV'
        dtf_name_hadr_psi_pv  = 'DTF_Psi_PV'

        particles[0].addTool( particles[0].DTF.clone( dtf_name_hadr_jpsi_pv,
                                              Verbose = True,
                                              constrainToOriginVertex = True,
                                              daughtersToConstrain = ["J/psi(1S)" ] ) )
        particles[0].ToolList += [ "TupleToolDecayTreeFitter/"+dtf_name_hadr_jpsi_pv ]

        particles[0].addTool( particles[0].DTF.clone( dtf_name_hadr_psi_pv,
                                              Verbose = True,
                                              constrainToOriginVertex = True,
                                              daughtersToConstrain = ["psi(2S)" ],
                                              Substitutions = psi2s_sub ) )
        particles[0].ToolList += [ "TupleToolDecayTreeFitter/"+dtf_name_hadr_psi_pv ]


    # Adding hop from loki functor
    from Configurables import  LoKi__Hybrid__TupleTool
    LoKi_HOP = LoKi__Hybrid__TupleTool("LoKi_HOP")
    LoKi_HOP.Variables ={
        'hop_LoKi_mass_bv': 'BPVHOPM()',
        'hop_LoKi_mass': 'HOPM(0,0,0)',
                }

    if 'e' in lepton_mode:
        particles[0].ToolList += [ "LoKi::Hybrid::TupleTool/LoKi_HOP" ]
        particles[0].addTool(LoKi_HOP)

    # Adding MIPCHI2 variable to p and pi
    LoKi_MIPCHI2 = LoKi__Hybrid__TupleTool("LoKi_MIPCHI2DV")
    LoKi_MIPCHI2.Variables ={
        "MIPCHI2" : "MIPCHI2DV(PRIMARY)",
                }

    # Adding Track momenta to l1 and l2
    LoKi_Track_mom = LoKi__Hybrid__TupleTool("LoKi_Track_mom")
    LoKi_Track_mom.Variables ={
        "TRACK_P"  : "PPINFO(504,-100,-200)",
        "TRACK_PT" : "PPINFO(505,-100,-200)",
        "TRACK_PX" : "PPINFO(505,-1e7,-1e7) * cos( PHI )",
        "TRACK_PY" : "PPINFO(505,-1e7,-1e7) * sin( PHI )",
        "TRACK_PZ" : "PPINFO(505,-1e7,-1e7) * sinh(ETA )",
                }

    #adding DOCA variables
    LoKi_DOCA = LoKi__Hybrid__TupleTool("LoKi_DOCA")
    LoKi_DOCA.Variables = {
        "DOCA01"  :  "DOCA(0,1)",
        "DOCA02"  :  "DOCA(0,2)",
        "DOCA12"  :  "DOCA(1,2)",
        "DOCA_MAX":  "DOCAMAX",
        }

    LoKi_ACC = LoKi__Hybrid__TupleTool("LoKi_ACC")
    LoKi_ACC.Variables = {
        "InAccSpd"  : "PPINFO( LHCb.ProtoParticle.InAccSpd,  -1 )",
        "InAccPrs"  : "PPINFO( LHCb.ProtoParticle.InAccPrs,  -1 )",
        "InAccEcal" : "PPINFO( LHCb.ProtoParticle.InAccEcal, -1 )",
        "InAccHcal" : "PPINFO( LHCb.ProtoParticle.InAccHcal, -1 )",
        "InAccMuon" : "PPINFO( LHCb.ProtoParticle.InAccMuon, -1 )",
        "ETA" : "ETA",
        "PHI" : "PHI"
        }

    for lept in leptons:
        lept.ToolList += [ "LoKi::Hybrid::TupleTool/LoKi_Track_mom" ]
        lept.addTool(LoKi_Track_mom)
        if 'e' in lepton_mode:
            lept.ToolList += ["TupleToolBremInfo"]

    for hadr in hadrons:
        hadr.ToolList += [ "LoKi::Hybrid::TupleTool/LoKi_MIPCHI2DV" ]
        hadr.addTool(LoKi_MIPCHI2)

    for res in resonances:
        res.ToolList += [ "LoKi::Hybrid::TupleTool/LoKi_DOCA"]
        res.addTool(LoKi_DOCA)

    for part in leptons+hadrons:
        part.ToolList += [ "LoKi::Hybrid::TupleTool/LoKi_ACC" ]
        part.addTool(LoKi_ACC)

    # Code by Maarten to include all kinds of q2 variables
    ep_pt = "CHILD(1, CHILD(1, PPINFO(505,-100,-200)))"
    em_pt = "CHILD(1, CHILD(2, PPINFO(505,-100,-200)))"
    ep_eta = "CHILD(1, CHILD(1, ETA))"
    em_eta = "CHILD(1, CHILD(2, ETA))"
    ep_phi = "CHILD(1, CHILD(1, PHI))"
    em_phi = "CHILD(1, CHILD(2, PHI))"

    
    particles[0].addTupleTool('LoKi::Hybrid::TupleTool/q2vars')
    particles[0].q2vars.Variables = { 
                               "q2"              : "(MAXTREE('J/psi(1S)'==ABSID,M)/GeV)**2" 
                             , "q2_nobrem"       : "(2.0*{0}*{1}*(cosh({2}-{3})-cos({4}-{5})))/((GeV)**2)".format(ep_pt,em_pt,ep_eta,em_eta,ep_phi,em_phi) 
                             }

    # Mass substitutions
    from Configurables import TupleToolSubMass
    particles[0].addTool( TupleToolSubMass )
    particles[0].ToolList += [ "TupleToolSubMass" ]
    particles[0].TupleToolSubMass.EndTreePIDs = [22]

    particles[0].TupleToolSubMass.Substitution       += [ "p+ => pi+" ]
    particles[0].TupleToolSubMass.Substitution       += [ "p+ => K+" ]
    particles[0].TupleToolSubMass.Substitution       += [ "pi- => p~-" ]
    particles[0].TupleToolSubMass.Substitution       += [ "pi- => K-" ]
    #e/mu for vetoes of ee/mumu resonances, emu swaps, hadronic vetoes 
    particles[0].TupleToolSubMass.DoubleSubstitution += [ "mu+/e- => e+/mu-" ]
    particles[0].TupleToolSubMass.DoubleSubstitution += [ "e+/e- => mu+/mu-" ]
    particles[0].TupleToolSubMass.DoubleSubstitution += [ "mu+/mu- => e+/e-" ]
    particles[0].TupleToolSubMass.DoubleSubstitution += [ "mu+/e- => e+/pi-" ]
    particles[0].TupleToolSubMass.DoubleSubstitution += [ "e+/e- => mu+/pi-" ]
    particles[0].TupleToolSubMass.DoubleSubstitution += [ "mu+/mu-=> e+/pi-" ]
    particles[0].TupleToolSubMass.DoubleSubstitution += [ "mu+/e- => pi+/mu-" ]
    particles[0].TupleToolSubMass.DoubleSubstitution += [ "e+/e- => pi+/mu-" ]
    particles[0].TupleToolSubMass.DoubleSubstitution += [ "mu+/mu- => pi+/e-" ]
    particles[0].TupleToolSubMass.DoubleSubstitution += [ "p+/pi- => pi+/p~-" ] 


    # Adding hop from tuple tool
    from Configurables import TupleToolHOP
    particles[0].addTool(TupleToolHOP, name = "LbHOP")
    particles[0].ToolList += [ "TupleToolHOP/LbHOP"]

    # Add isolation variables
    from Configurables import TupleToolTrackIsolation,TupleToolConeIsolation
    if not is_mdst:
        particles[0].addTool(TupleToolTrackIsolation, name = "TrackIsoInfo")
        particles[0].addTool(TupleToolConeIsolation, name = "ConeIsoInfo")

        particles[0].ToolList += ["TupleToolTrackIsolation/TrackIsoInfo","TupleToolConeIsolation/ConeIsoInfo"]
        particles[0].TrackIsoInfo.MaxConeAngle = 0.5
        particles[0].TrackIsoInfo.Verbose = True
        particles[0].ConeIsoInfo.MinConeSize = 0.5 # ONLY USE ANGLE OF 0.5
        particles[0].ConeIsoInfo.MaxConeSize = 0.5 # ONLY USE ANGLE OF 0.5
        particles[0].ConeIsoInfo.Verbose = True

    # Set correct name depending on ee or mumu
    if 'ee' in lepton_mode:
        loki_name = 'JPsEE'
    else:
        loki_name = 'JPsMM'
    LoKi_Tool = getLoKiTool(loki_name, location_list[0],is_mdst)
    particles[0].ToolList += ["LoKi::Hybrid::TupleTool/LoKi_Tool{}".format(loki_name)]
    particles[0].addTool(LoKi_Tool)

    if is_mc:
        particles[0].ToolList += [
            'TupleToolMCBackgroundInfo'
        ]

        # Add MC tuple tools
        mctruth = dtt.addTupleTool('TupleToolMCTruth')
        dtt.ToolList += ['TupleToolMCTruth']
        dtt.TupleToolMCTruth.ToolList += [
            'MCTupleToolKinematic',
            'MCTupleToolHierarchy',
            'MCTupleToolPID',
            'MCTupleToolReconstructed'
        ]
        particles[0].addTupleTool('TupleToolMCTruth')
        particles[0].ToolList += ['TupleToolMCTruth']

    DaVinci().UserAlgorithms += [dtt]
   # tuple_seq = SelectionSequence('{}_SelSeq'.format(tuple_name), tuple_selection)

    return True #tuple_seq.sequence()


def tuple_sequence():
    """Return the sequencer used for running tuple algorithms.

    The sequencer is configured such that all members are run, as their filter
    flags are ignored.
    """
    seq = GaudiSequencer('TupleSeq')
    seq.IgnoreFilterPassed = True

    return seq



# FILE CONTAINING ALL DECAY DESCRIPTORS

dict_rec = {
    'Lb2JpsiL_mm'  :'[Lambda_b0 ->   ^(J/psi(1S) -> ^mu+  ^mu-)   ^(Lambda0 -> ^p+ ^pi-)]CC',
    'Lb2JpsiL_ee'  :'[Lambda_b0 ->   ^(J/psi(1S) -> ^e+  ^e- )    ^(Lambda0 -> ^p+ ^pi-)]CC',
    'Lb2Lmm_SS'    :'[Lambda_b0 ->   ^(J/psi(1S) -> ^mu+  ^mu+)   ^(Lambda0 -> ^p+ ^pi-)]CC',
    'Lb2Lee_SS'    :'[Lambda_b0 ->   ^(J/psi(1S) -> ^e+  ^e+ )    ^(Lambda0 -> ^p+ ^pi-)]CC',
    'Lb2Lemu'      :"[Lambda_b0 -> [ ^(J/psi(1S) -> ^e+  ^mu-)]CC ^(Lambda0 -> ^p+ ^pi-)]CC",
    'Lb2Lemu_SS'   :"[Lambda_b0 -> [ ^(J/psi(1S) -> ^e+  ^mu-)]CC ^(Lambda0 -> ^p+ ^pi-)]CC",
    'Bd2JpsiKs_mm' :'        B0 ->   ^(J/psi(1S) -> ^mu+  ^mu-)   ^(KS0     -> ^pi+ ^pi-)   ',
    'Bd2JpsiKs_ee' :'        B0 ->   ^(J/psi(1S) -> ^e+  ^e- )    ^(KS0     -> ^pi+ ^pi-)   ',
    'Bd2Ksmm_SS'   :'        B0 ->   ^(J/psi(1S) -> ^mu+  ^mu+)   ^(KS0     -> ^pi+ ^pi-)   ',
    'Bd2Ksee_SS'   :'        B0 ->   ^(J/psi(1S) -> ^e+  ^e+ )    ^(KS0     -> ^pi+ ^pi-)   ',
    'Bd2Ksemu   '  :"        B0 -> [ ^(J/psi(1S) -> ^e+  ^mu-)]CC ^(KS0     -> ^pi+ ^pi-)   ",
    'Bd2Ksemu_SS'  :"        B0 -> [ ^(J/psi(1S) -> ^e+  ^mu+)]CC ^(KS0     -> ^pi+ ^pi-)   ",
    'Bu2JpsiK_mm'  :'[       B+ ->   ^(J/psi(1S) -> ^mu+  ^mu-)   ^K+                   ]CC',
    'Bu2JpsiK_ee'  :'[       B+ ->   ^(J/psi(1S) -> ^e+  ^e- )    ^K+                   ]CC',
    'Bu2Kmm_SS'    :'[       B+ ->   ^(J/psi(1S) -> ^mu+  ^mu+)   ^K+                   ]CC',
    'Bu2Kee_SS'    :'[       B+ ->   ^(J/psi(1S) -> ^e+  ^e+ )    ^K+                   ]CC',
    'Bu2Kemu   '   :"[       B+ -> [ ^(J/psi(1S) -> ^e+  ^mu-)]CC ^K+                   ]CC",
    'Bu2Kemu_SS'   :"[       B+ -> [ ^(J/psi(1S) -> ^e+  ^mu+)]CC ^K+                   ]CC",
}

dict_mc = {
    'Lb2LJpsmm'    :'[Lambda_b0 =>  ^(J/psi(1S) => ^mu+  ^mu-)    ^(Lambda0 => ^p+ ^pi-)]CC',
    'Lb2LJpsee'    :'[Lambda_b0 =>  ^(J/psi(1S) => ^e+  ^e- )    ^(Lambda0 => ^p+ ^pi-)]CC',
    'Lb2LPsimm'    :'[Lambda_b0 =>  ^(  psi(2S) => ^mu+  ^mu-)    ^(Lambda0 => ^p+ ^pi-)]CC',
    'Lb2LPsiee'    :'[Lambda_b0 =>  ^(  psi(2S) => ^e+  ^e- )    ^(Lambda0 => ^p+ ^pi-)]CC',
    'Lb2Lee'       :'[Lambda_b0 =>                 ^e+  ^e-      ^(Lambda0 => ^p+ ^pi-)]CC',
    'Lb2Lmm'       :'[Lambda_b0 =>                 ^mu+  ^mu-     ^(Lambda0 => ^p+ ^pi-)]CC',
    'Lb2Lem'       :"([Lambda_b0 =>                 ^e+  ^mu-      ^(Lambda0 => ^p+ ^pi-)]CC) || ([Lambda_b0 =>  ^e-  ^mu+ ^(Lambda0 => ^p+ ^pi-)]CC)",
    'Bd2Ksmm'      :'    [B0]cc =>                 ^mu+  ^mu-     ^(KS0     => ^pi+ ^pi-)   ',
    'Bd2Ksee'      :'    [B0]cc =>                 ^e+  ^e-      ^(KS0     => ^pi+ ^pi-)   ',
    'Bd2KsJpsmm'   :'    [B0]cc =>  ^(J/psi(1S) => ^mu+  ^mu-)    ^(KS0     => ^pi+ ^pi-)   ',
    'Bd2KsJpsee'   :'    [B0]cc =>  ^(J/psi(1S) => ^e+  ^e- )    ^(KS0     => ^pi+ ^pi-)   ',
    'Bd2KsPsimm'   :'    [B0]cc =>  ^(  psi(2S) => ^mu+  ^mu-)    ^(KS0     => ^pi+ ^pi-)   ',
    'Bd2KsPsiee'   :'    [B0]cc =>  ^(  psi(2S) => ^e+  ^e- )    ^(KS0     => ^pi+ ^pi-)   ',
    'Bd2Ksemu'     :"    [B0]cc =>                  ^e+  ^mu-      ^(KS0   => ^pi+ ^pi-)  ||      [B0]cc =>  ^e-  ^mu+ ^(KS0    => ^pi+ ^pi-)    ",
    'Bu2JpsiK_mm'  :'[       B+ =>  ^(J/psi(1S) => ^mu+  ^mu-)    ^K+                   ]CC',
    'Bu2JpsiK_ee'  :'[       B+ =>  ^(J/psi(1S) => ^e+  ^e- )    ^K+                   ]CC',
    'Bu2PsiK_mm'   :'[       B+ =>  ^(  psi(2S) => ^mu+  ^mu-)    ^K+                   ]CC',
    'Bu2PsiK_ee'   :'[       B+ =>  ^(  psi(2S) => ^e+  ^e- )    ^K+                   ]CC',
    'Bu2Kemu'      :"([      B+ =>                 ^e+  ^mu-     ^K+                   ]CC) || ([       B+ =>  ^e-  ^mu+ ^K+                   ]CC)",

}

dict_mc_noV0dec = {k : ( v if 'Bu' in k
                 else v.replace('=> ^pi+ ^pi-)','').replace('(KS0','KS0') if 'Bd' in k
                 else v.replace('=> ^p+ ^pi-)' ,'').replace('(Lam','Lam') )
                 for (k,v) in dict_mc.items() }
print dict_mc_noV0dec
for keys,vals in dict_mc_noV0dec.iteritems():
    print keys,vals

def tuple_branches(decay_desc_full):
    # Now add resonances for data/MC and different b hadrons
    tuple_branches = []
    particles      = []

    # Some configuration
    charm_resonances = ['J/psi(1S)','psi(2S)']
    V0_resonances    = ['Lambda0'  ,'KS0'    ]

    V0dec = ('pi' in decay_desc_full) # Needed to add branches

    Lb_decay = ('Lambda' in decay_desc_full)
    Bd_decay = ('B0'     in decay_desc_full)
    Bu_decay = ('B+'     in decay_desc_full)

    # First add the b and s hadrons
    if Lb_decay: 
        tuple_branches = ['Lb','L0']
        particles     += ['Lambda0']
    elif Bd_decay:
        tuple_branches = ['B','K0'] 
        particles     += ['KS0']
    elif Bu_decay: 
        tuple_branches = ['B','K']  
        particles     += ['K+']

    # take leptons from decay descriptor
    # decay descriptor has first lepton after first or second ^ 
    # ['e+ ','mu+)    '] is an example string
    first_lepton = 1 

    # Add Jpsi if needed and appropriate leptons
    for charm_resonance in ['J/psi(1S)','psi(2S)']:
        if charm_resonance in decay_desc_full: # FOR DecayTree or MCTree with Jpsi
            tuple_branches += ['JPs']
            particles += [charm_resonance]        
            first_lepton += 1

    # Format leptons for SS parsing
    leptons = decay_desc_full.split('^')[first_lepton:first_lepton+2]
    print ('leptons',leptons)
    leptons[0] = leptons[0].replace(' ','')
    leptons[1] = leptons[1].split(')')[0].replace(' ','')
    print ('leptons',leptons)

    # Add lepton branches
    tuple_branches += ['L1','L2']    
    particles += leptons

    # Check if SS tuple, used to add correct L2 matching
    SS_tuple = (leptons in [['e+','e+'],['e+','mu+'],['mu+','mu+']])

    # Add hadrons if the Lambda decays
    if V0dec:
        if Lb_decay:
            tuple_branches += ['P'  ,'Pi' ]
            particles += ['p+','pi-']
        elif Bd_decay:
            tuple_branches += ['Pi1','Pi2']
            particles += ['pi+','pi-']

    # Replace hats to get individual particles
    decay_desc = decay_desc_full.replace('^','') # remove hats because only one will be needed
    decay_desc_list = decay_desc.split('||') # TO DEAL WITH emu case: split e+mu- and e-mu+

    print particles
    print tuple_branches


    # add single hats
    branches = { tuple_branches[0] : decay_desc  }
    for i in range(len(tuple_branches)-1):
        print i
        part = particles[i]

        if part in charm_resonances or (part in V0_resonances and V0dec): 
            part = '('+part # To replace (Lambda for ex.

        # Run over separate decay_desc to deal with e+mu- and e-mu+ case
        decay_desc_replaced = []
        for j in range(len(decay_desc_list)): # For each separate list, replace
            if j == 0:
                decay_desc_replaced.append(decay_desc_list[j].replace(' '+part,'^'+part,1) )
                decay_desc_tot = decay_desc_replaced[j]
                print decay_desc_tot

            else:     
                if 'e' in part or 'mu' in part:
                    if '-' in part: part = part.replace('-','+')
                    else:           part = part.replace('+','-') # Swap charge for opposite charge lepton pair
                print part
                print decay_desc_list[j]
                decay_desc_replaced.append(decay_desc_list[j].replace(' '+part,'^'+part,1) )
                decay_desc_tot = "{0} || {1}".format(decay_desc_tot,decay_desc_replaced[j])

        branches[tuple_branches[i+1]] = decay_desc_tot

    if SS_tuple: #IF SAME SIGN IN TUPLE BRANCHES 
        # decay desc has ^e+   e+ twice
        print branches
        input_string  = '^{0}  {1}'.format(leptons[0],leptons[1])
        output_string = ' {0} ^{1}'.format(leptons[0],leptons[1])
        print input_string,output_string
        branches['L2'] = branches['L2'].replace(input_string,output_string)

    return branches,tuple_branches




from __future__ import print_function

from Configurables import (
    DaVinci,
    GaudiSequencer,
    EventNodeKiller,
    ProcStatusCheck
)
from StrippingConf.Configuration import (
    StrippingConf,
    StrippingStream
)
from StrippingSettings.Utils import strippingConfiguration
from StrippingArchive.Utils import buildStreams
from StrippingArchive import strippingArchive

from DSTWriters.microdstelements import *
from DSTWriters.Configuration import (
    SelDSTWriter,
    stripDSTStreamConf,
    stripDSTElements
)

STRIPPING_VERSIONS = {
    '2011': 'Stripping21r1p2',
    '2012': 'Stripping21r0p2',
    '2015': 'Stripping24r2',
    '2016': 'Stripping28r2',
    '2017': 'Stripping29r2p1',
    '2018': 'Stripping34r0p1'
}

def stripping(data_type, stripping_lines):
    # kill the banks
    event_node_killer = EventNodeKiller('StripKiller')
    event_node_killer.Nodes = ['/Event/AllStreams', '/Event/Strip']

    # figure out stripping version from the year
    stripping_version = STRIPPING_VERSIONS[data_type]
    # build streams
    streams = buildStreams(stripping=strippingConfiguration(stripping_version),
                           archive=strippingArchive(stripping_version))
    # declare your custom stream
    custom_stream = StrippingStream('AllStreams')

    for stream in streams:
        for sline in stream.lines:
            if sline.name() in stripping_lines:
                sline._prescale = 1.0
                custom_stream.appendLines([sline])

    filterBadEvents = ProcStatusCheck()

    # configure your custom stream
    sc = StrippingConf(Streams=[custom_stream],
                       MaxCandidates=2000,
                       AcceptBadEvents=False,
                       BadEventSelection=filterBadEvents)

    enablePacking = False

    SelDSTWriterElements = {'default': stripDSTElements(pack=enablePacking)}

    SelDSTWriterConf = {'default': stripDSTStreamConf(pack=enablePacking,
                                                      selectiveRawEvent=True,
                                                      fileExtension='.RD_LBTOL0LL.DST')}

    dstWriter = SelDSTWriter('MyDSTWriter',
                             StreamConf=SelDSTWriterConf,
                             MicroDSTElements=SelDSTWriterElements,
                             OutputFileSuffix='',
                             SelectionSequences=sc.activeStreams()
                             )

    DaVinci().ProductionType = 'Stripping'
    seq = GaudiSequencer('TupleSeq')
    seq.IgnoreFilterPassed = True
    seq.Members += [event_node_killer, sc.sequence()] + [dstWriter.sequence()]

    # This is a hack to work around https://gitlab.cern.ch/lhcb-dirac/LHCbDIRAC/merge_requests/736/
    if getattr(stripping, 'NEED_FILENAME_PATCH', True):
        import atexit
        atexit.register(fix_filenames)
        stripping.NEED_FILENAME_PATCH = False

    return seq


def fix_filenames():
    """This is a hack to work around https://gitlab.cern.ch/lhcb-dirac/LHCbDIRAC/merge_requests/736/"""
    import glob
    import os
    for fn in glob.glob('*.anaprod_bstojpsiphi.ldst'):
        print('Renaming', fn, 'to', fn.lower())
        os.rename(fn, fn.lower())

        for xml_fn in glob.glob('summaryDaVinci_*.xml'):
            print('Fixing XML in', xml_fn)
            with open(xml_fn, 'rt') as fp:
                xml = fp.read()
            xml = xml.replace(fn, fn.lower())
            with open(xml_fn, 'wt') as fp:
                fp.write(xml)

        print('Fixing pool_xml_catalog.xml')
        with open('pool_xml_catalog.xml', 'rt') as fp:
            xml = fp.read()
        xml = xml.replace(fn, fn.lower())
        with open('pool_xml_catalog.xml', 'wt') as fp:
            fp.write(xml)



import os

from Configurables import (
    BTaggingTool,
    DaVinci,
    GaudiSequencer,
    MCDecayTreeTuple,
    MCTupleToolP2VV
)

from FlavourTagging.Tunings import applyTuning as applyFTTuning
from PhysSelPython.Wrappers import (
    AutomaticData,
    FilterSelection,
    SelectionSequence,
    TupleSelection
)


L0_LINES = [
    'L0DiMuonDecision',
    'L0GlobalDecision',
    'L0MuonDecision'
]
HLT1_LINES = [
    'Hlt1DiMuonHighMassDecision',
    'Hlt1DiMuonLowMassDecision',
    'Hlt1DiMuonNoL0Decision',
    'Hlt1GlobalDecision',
    'Hlt1L0AnyDecision',
    'Hlt1SingleMuonHighPTDecision',
    'Hlt1TrackMVADecision',
    'Hlt1TrackMVALooseDecision',
    'Hlt1TrackMuonDecision',
    'Hlt1TrackMuonMVADecision',
    'Hlt1TwoTrackMVADecision',
    'Hlt1TwoTrackMVALooseDecision'
]
HLT2_LINES = [
    'Hlt2DiMuonDecision',
    'Hlt2DiMuonDetachedDecision',
    'Hlt2DiMuonDetachedHeavyDecision',
    'Hlt2DiMuonDetachedJPsiDecision',
    'Hlt2DiMuonDetachedPsi2SDecision',
    'Hlt2DiMuonJPsiDecision',
    'Hlt2SingleMuonDecision',
    'Hlt2SingleMuonHighPTDecision',
    'Hlt2SingleMuonLowPTDecision',
    'Hlt2SingleMuonRareDecision',
    'Hlt2SingleMuonVHighPTDecision',
    'Hlt2Topo2BodyDecision',
    'Hlt2Topo3BodyDecision',
    'Hlt2Topo4BodyDecision',
    'Hlt2TopoMu2BodyDecision',
    'Hlt2TopoMu3BodyDecision',
    'Hlt2TopoMu4BodyDecision',
    'Hlt2TopoMuMu2BodyDecision',
    'Hlt2TopoMuMu3BodyDecision',
    'Hlt2TopoMuMu4BodyDecision'
]
TRIGGER_LINES = L0_LINES + HLT1_LINES + HLT2_LINES


def tuple_maker(name, decay, branches, stripping_line, is_mc, input_type, refit_pvs=True):
    """Return a sequence for producing ntuples.

    Assumes the decay structure is like:

        Beauty -> (J/psi -> mu- mu+) X

    The `name` arg must start with Bu, Bd, or Bs. This is used to
    determine the head of the decay.
    """
    assert name.startswith(('Bu', 'Bd', 'Bs')), 'Unknown decay head in `name`'
    # Parse the name to get the decay head
    decay_head = {
        'Bu': 'B+',
        'Bd': 'B0',
        'Bs': 'B_s0'
    }[name[:2]]
    # Assume we're processing a P2VV decay for neutral B's, in which case we'll
    # add some extra tools later
    p2vv = decay_head in ['B0', 'B_s0']

    # TODO: consider moving this block out of this method
    is_mdst = input_type.upper() == 'MDST'
    # Don't get the tagging decision on MC microDSTs
    with_ft = not (is_mc and is_mdst)
    tes_root = ''
    if is_mc:
        tes_root = '/Event/AllStreams'
    else:
        if is_mdst:
            tes_root = '/Event/Leptonic'
        else:
            tes_root = '/Event/Dimuon'
    if is_mdst:
        DaVinci().RootInTES = tes_root
    location = 'Phys/{}/Particles'.format(stripping_line)
    if not is_mdst:
        location = os.path.join(tes_root, location)

    tuple_input = AutomaticData(location)
    # Require that there are valid PVs after refitting, as required by
    # TupleToolPropertime
    if refit_pvs:
        tuple_input = FilterSelection(
            'CheckRefittedPVs_{}'.format(name),
            [tuple_input],
            Code='BPVVALID()',
            ReFitPVs=True
        )

    tools = [
        'TupleToolANNPID',
        'TupleToolEventInfo',
        'TupleToolKinematic',
        'TupleToolL0Calo',
        'TupleToolL0Data',
        'TupleToolPid',
        'TupleToolPrimaries',
        'TupleToolPropertime',
        'TupleToolTISTOS'
    ]
    if is_mc:
        tools += [
            'TupleToolMCBackgroundInfo'
        ]

    tuple_selection = TupleSelection(
        '{}_Tuple'.format(name),
        [tuple_input],
        decay,
        ToolList=tools,
        ReFitPVs=refit_pvs
    )
    dtt = tuple_selection.algorithm()
    dtt.addBranches(branches)

    geom = dtt.addTupleTool('TupleToolGeometry')
    geom.RefitPVs = refit_pvs
    geom.Verbose = True

    reco_stats = dtt.addTupleTool('TupleToolRecoStats')
    reco_stats.Verbose = True

    track_inf = dtt.addTupleTool('TupleToolTrackInfo')
    track_inf.Verbose = True

    track_pos = dtt.addTupleTool('TupleToolTrackPosition')
    track_pos.Z = 7500.0

    trig_tool = dtt.addTupleTool('TupleToolTrigger')
    trig_tool.Verbose = True
    trig_tool.TriggerList = TRIGGER_LINES
    trig_tool.OutputLevel = 6

    b_tistos = dtt.B.addTupleTool('TupleToolTISTOS')
    b_tistos.Verbose = True
    b_tistos.TriggerList = TRIGGER_LINES

    jpsi_tistos = dtt.Jpsi.addTupleTool('TupleToolTISTOS')
    jpsi_tistos.Verbose = True
    jpsi_tistos.TriggerList = TRIGGER_LINES

    LoKi_B = dtt.B.addTupleTool('LoKi::Hybrid::TupleTool/LoKi_B')
    LoKi_B.Variables = {
        # TODO: do we really need this? I'm pretty sure this is just equivalent
        # to the B mass. Otherwise, will need to know the B decay products
        # 'JpsiPhiMass': 'WM(\'J/psi(1S)\',\'phi(1020)\')',
        'ETA': 'ETA',
        'DOCA': 'DOCA(1,2)',
        'Y': 'Y',
        'LV01': 'LV01',
        'LV02': 'LV02',
        'LOKI_FDCHI2': 'BPVVDCHI2',
        'LOKI_FDS': 'BPVDLS',
        'LOKI_DIRA': 'BPVDIRA',
        'LOKI_DTF_CTAU': 'DTF_CTAU(0, True)',
        'LOKI_DTF_CTAUS': 'DTF_CTAUSIGNIFICANCE(0, True)',
        'LOKI_DTF_CHI2NDOF': 'DTF_CHI2NDOF(True)',
        'LOKI_DTF_CTAUERR': 'DTF_CTAUERR(0, True)',
        'LOKI_MASS_JpsiConstr': 'DTF_FUN(M, True, \'J/psi(1S)\')',
        'LOKI_DTF_VCHI2NDOF': 'DTF_FUN(VFASPF(VCHI2/VDOF), True)'
    }

    LoKi_Jpsi = dtt.Jpsi.addTupleTool('LoKi::Hybrid::TupleTool/LoKi_Jpsi')
    LoKi_Jpsi.Variables = {
        'ETA': 'ETA',
        'Y': 'Y',
        'LV01': 'LV01',
        'LV02': 'LV02',
        'LOKI_FDCHI2': 'BPVVDCHI2',
        'LOKI_FDS': 'BPVDLS',
        'LOKI_DIRA': 'BPVDIRA'
    }

    LoKi_muonp = dtt.muplus.addTupleTool('LoKi::Hybrid::TupleTool/LoKi_muonp')
    LoKi_muonm = dtt.muminus.addTupleTool('LoKi::Hybrid::TupleTool/LoKi_muonm')
    LoKi_muonp.Variables = LoKi_muonm.Variables = {
        'LOKI_ETA': 'ETA',
        'LOKI_Y': 'Y'
    }

    # refit with constraints
    fitter_params = [
        ('ConstJpsi', True, ['J/psi(1S)']),
        ('ConstJpsiNoPV', False, ['J/psi(1S)']),
        ('ConstBJpsi', True, [decay_head, 'J/psi(1S)']),
        ('ConstOnlyPV', True, [])
    ]
    # Only the Bd2JpsiKstar tuple has this fitter configuration
    if decay_head == 'B0':
        fitter_params += [
            ('ConstBJpsiNoPV', False, [decay_head, 'J/psi(1S)'])
        ]
    for fitter_name, constrain_to_pv, daughter_constraints in fitter_params:
        fitter = dtt.B.addTupleTool('TupleToolDecayTreeFitter/{}'.format(fitter_name))
        fitter.Verbose = True
        fitter.UpdateDaughters = True
        fitter.constrainToOriginVertex = constrain_to_pv
        fitter.daughtersToConstrain = daughter_constraints

    if with_ft:
        btag = dtt.addTupleTool('TupleToolTagging')
        # Take the tagging decision from microDSTs, otherwise recompute it
        if is_mdst:
            btag.UseFTfromDST = True
        else:
            btag.Verbose = True
            btag.AddMVAFeatureInfo = True

            btagtool = btag.addTool(BTaggingTool, name='MyBTaggingTool_{}'.format(name))

            applyFTTuning(btagtool, tuning_version='Summer2017Optimisation_v4_Run2')
            btag.TaggingToolName = btagtool.getFullName()

    LoKi_EvtTuple = dtt.addTupleTool('LoKi::Hybrid::EvtTupleTool/LoKi_EvtTuple')
    LoKi_EvtTuple.VOID_Variables = {
        'LoKi_nPVs': "CONTAINS('Rec/Vertex/Primary')",
        'LoKi_nSpdMult': "CONTAINS('Raw/Spd/Digits')",
        'LoKi_nVeloClusters': "CONTAINS('Raw/Velo/Clusters')",
        'LoKi_nVeloLiteClusters': "CONTAINS('Raw/Velo/LiteClusters')",
        'LoKi_nITClusters': "CONTAINS('Raw/IT/Clusters')",
        'LoKi_nTTClusters': "CONTAINS('Raw/TT/Clusters')",
        'LoKi_nOThits': "CONTAINS('Raw/OT/Times')"
    }

    tuple_seq = SelectionSequence('{}_SelSeq'.format(name), tuple_selection)

    if is_mc:
        # Add MC tuple tools
        mctruth = dtt.addTupleTool('TupleToolMCTruth')
        mctruth.ToolList = [
            'MCTupleToolKinematic',
            'MCTupleToolHierarchy',
            'MCTupleToolPID'
        ]

    return tuple_seq.sequence()


def mc_tuple_maker(name, decay, branches):
    """Return a configured MCDecayTreeTuple instance."""
    dtt = MCDecayTreeTuple('{}_MCTuple'.format(name))
    dtt.Decay = decay
    dtt.TupleName = 'MCTuple'
    dtt.ToolList += [
        'MCTupleToolKinematic',
        'TupleToolEventInfo',
        'MCTupleToolHierarchy',
        'MCTupleToolPrimaries',
        'MCTupleToolPID'
    ]
    dtt.addBranches(branches)

    return dtt


def tuple_sequence():
    """Return the sequencer used for running tuple algorithms.

    The sequencer is configured such that all members are run, as their filter
    flags are ignored.
    """
    seq = GaudiSequencer('TupleSeq')
    seq.IgnoreFilterPassed = True

    return seq



###############################################################################
# (c) Copyright 2020 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
import os
from os.path import isfile, join

import pytest
from click.testing import CliRunner

import LbAPLocal
import LbAPLocal.testing
from fixture_loader import discover_fixtures
from LbAPLocal.cli import legacy_main as main

# ============================================================================
# CWL-based tests (primary) — auto-discovered from tests/fixtures/
# ============================================================================


def _cwl_test_params():
    """Auto-discover all fixtures and generate test parameters."""
    params = []
    for prod_name, fixture_set in discover_fixtures().items():
        for job_name in fixture_set.job_names:
            params.append(
                pytest.param(prod_name, job_name, id=f"{prod_name}/{job_name}")
            )
    return params


@pytest.mark.slow
@pytest.mark.parametrize(
    "cwl_fixture_set,job_name", _cwl_test_params(), indirect=["cwl_fixture_set"]
)
def test_cwl_test(
    data_pkg_repo, mock_dirac_calls, cwl_fixture_set, job_name, tmp_path, monkeypatch
):
    """Test the CWL-based execution path using LbAPCommon + dirac-cwl-run."""
    import typer

    from LbAPLocal.cli.main import cwl_test

    monkeypatch.setattr(
        LbAPLocal.cli,
        "get_auth_headers",
        lambda: dict(headers={"Authorization": "Bearer "}),
    )

    output_dir = tmp_path / "cwl_output"

    try:
        cwl_test(
            production_name=cwl_fixture_set.production_name,
            job_name=job_name,
            skip_validation=True,
            output_dir=output_dir,
            n_lfns=1,
        )
    except (typer.Exit, SystemExit) as e:
        exit_code = getattr(e, "exit_code", getattr(e, "code", 1))
        pytest.fail(f"cwl_test exited with code {exit_code}")

    # CWL workflow was generated
    cwl_files = list(output_dir.glob("*.cwl"))
    assert cwl_files, "No CWL workflow file generated"

    # Inputs and replica map were written by mock_dirac_calls
    inputs_files = list(output_dir.glob("*-inputs.yml"))
    assert inputs_files, "No CWL inputs file generated"
    replica_maps = list(output_dir.glob("*-replica-map.json"))
    assert replica_maps, "No replica map generated"

    # CWL output directory exists (runner executed)
    cwl_output = output_dir / "cwl_output"
    assert cwl_output.is_dir(), "CWL output directory not created"


# ============================================================================
# Legacy tests (deprecated, kept temporarily)
# ============================================================================


@pytest.mark.skip(reason="Legacy lb-prod-run path, replaced by test_cwl_test")
@pytest.mark.slow
@pytest.mark.vcr
@pytest.mark.parametrize(
    "name,n_events",
    [
        ("2016_MagDown_PromptMC_D02KK", 2444),
        ("2016_MagUp_PromptMC_D02KK", 2393),
        ("2016_MagDown_PromptMC_D02KK_noautoconf", 2444),
        ("2016_MagUp_PromptMC_D02KK_noautoconf", 2393),
    ],
)
def test_run_test(
    with_proxy, data_pkg_repo, with_data, name, n_events, monkeypatch, record_mode
):
    if record_mode == "none":
        monkeypatch.setattr(
            LbAPLocal.cli,
            "get_auth_headers",
            lambda: dict(headers={"Authorization": "Bearer "}),
        )
        monkeypatch.setattr(
            LbAPLocal.testing,
            "get_auth_headers",
            lambda: dict(headers={"Authorization": "Bearer "}),
        )

    runner = CliRunner()
    result = runner.invoke(main, ["test", "executabletests", name, "--no-validate"])
    assert result.exit_code == 0, result.stdout

    assert "Application Manager Finalized successfully" in result.stdout
    assert "Application Manager Terminated successfully" in result.stdout
    assert f"{n_events} events processed" in result.stdout
    assert "Summary of log messages" in result.stdout
    assert "General explanations" in result.stdout
    assert "Histograms are not being saved" in result.stdout

    assert "Output can be found in" in result.stdout
    output_dir = result.stdout.split("Output can be found in")[-1].strip()

    output_fn = join(output_dir, "00012345_00006789_1.D02KK.ROOT")
    assert isfile(output_fn)
    assert 100 * 1024 <= os.stat(output_fn).st_size <= 1024**2

    stdout_fn = join(output_dir, "stdout.log")
    assert isfile(stdout_fn)
    with open(stdout_fn, "rt") as fp:
        stdout = fp.read()
    assert "Application Manager Finalized successfully" in stdout
    assert "Application Manager Terminated successfully" in stdout
    assert f"{n_events} events processed" in stdout

    stderr_fn = join(output_dir, "stderr.log")
    assert isfile(stderr_fn)
